/*
 * FindDataAroundXY.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.app;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.host.*;
import misao.pixy.*;

public class FindDataAroundXY {
    public static void main ( String[] args ) throws Exception {	
	System.out.println("Output data around (x,y).");

	String pxf_filename = "";
	String xy_str = "";

	if (args.length >= 2) {
	    pxf_filename = args[0];
	    xy_str = args[1];
	} else {
	    DataInputStream din = new DataInputStream(System.in);

	    System.out.print("PXF file: ");
	    pxf_filename = din.readLine();

	    System.out.print("(x,y): ");
	    System.out.println("  ex. (x,y) = ( 12.34, 567.89)");
	    System.out.println("  acceptable style: 12.34 567.89");
	    System.out.print("(x,y): ");
	    xy_str = din.readLine();
	}

	StringTokenizer st = new StringTokenizer(xy_str);
	double x = Astro.doubleValueOf(st.nextToken());
	double y = Astro.doubleValueOf(st.nextToken());
	Position pos = new Position(x,y);

	FindDataAroundXY.operate(pxf_filename, pos);
    }

    private static void operate ( String pxf_filename, Position pos ) throws Exception {
	File file = new File(pxf_filename);
	FileInputStream fis = new FileInputStream(file);
	DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
	PxfFile pxf = new PxfFile();
	pxf.read(dis);
	dis.close();

	Position df_pos = pxf.image_info.distortion_field.getValue(pos);
	Position p = new Position(pos.getX() + df_pos.getX() - (double)pxf.image_info.image_size.getWidth() / 2.0, pos.getY() + df_pos.getY() - (double)pxf.image_info.image_size.getHeight() / 2.0);
	MapFunction mf = new MapFunction(1.0, 360.0 - pxf.image_info.pa, new Position());
	p = mf.map(p);
	p.rescale(pxf.image_info.field_of_view / (double)pxf.image_info.image_size.getWidth());
	CoorMapping coor = new CoorMapping(pxf.image_info.center_coor, p);
	System.out.println(coor.toStringRA() + "  " + coor.toStringDecl());

	Vector list = pxf.pair_list.getAllList();
	for (int k = 0 ; k < list.size() ; k++) {
	    Vector l = (Vector)list.elementAt(k);

	    for (int j = 0 ; j < l.size() ; j++) {
		PairData pair = (PairData)l.elementAt(j);
		StarData sdd = (StarData)pair.getFirstData();
		StarData sdc = (StarData)pair.getSecondData();
		StarData sd = sdc;
		if (sd == null)
		    sd = sdd;
		p = sd.getPosition();
		if (sdd != null)
		    p = sdd.getPosition();

		if (Math.abs(p.getX() - pos.getX()) < 3.0  &&  Math.abs(p.getY() - pos.getY()) < 3.0) {
		    System.out.println(pair.getName());
		    Vector alias_list = pair.getAliasList();
		    for (int n = 0 ; n < alias_list.size() ; n++) {
			ReferenceName name = (ReferenceName)alias_list.elementAt(n);
			System.out.println(name.getType().toString() + "=" + name.getName());
		    }
		    pair.print(System.out);
		    System.out.println("");
		}
	    }
	}
    }
}
