/*
 * NDChecker.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.database;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.pixy.*;
import misao.host.*;

public class NDChecker implements Printable {
    File top_directory;

    DetectionCheckTable dc_table;
    Vector info_list;

    boolean output_midway_progress;
    boolean output_to_console;
    String output_file;

    public NDChecker ( File dir ) {
	top_directory = dir;

	dc_table = new DetectionCheckTable(3.0, 1.0);
	info_list = new Vector();

	output_midway_progress = true;
	output_to_console = true;
	output_file = null;
    }

    public void outputMidwayProgress ( boolean f ) {
	output_midway_progress = f;
    }

    public void outputToConsole ( boolean f ) {
	output_to_console = f;
    }

    public void outputToFile ( String s ) {
	output_file = s;
    }

    public void setPxfPath ( String[] pxf_path ) {
	dc_table.setPxfPath(pxf_path);
    }

    public void operate ( ) throws Exception {
	checkCurrentPxf();
	dc_table.removeStarsWithOnlyOneRecord();

	dc_table.setIgnoreImage(info_list);

	checkMagnitudeDatabase();

	checkPastPxf();

	PrintManager print_manager = new PrintManager();
	print_manager.outputToConsole(output_to_console);
	if (output_file != null  &&  output_file.length() > 0)
	    print_manager.openFile(output_file);
	print_manager.printObject(this);
	print_manager.close();
    }

    private void checkCurrentPxf ( ) throws Exception {
	readCurrentPxf(top_directory);
    }

    private void readCurrentPxf ( File directory ) throws Exception {
	String[] file_list = directory.list();
	for (int i = 0 ; i < file_list.length ; i++) {
	    file_list[i] = directory.getPath() + "/" + file_list[i];

	    File f = new File(file_list[i]);
	    if (f.isDirectory()) {
		readCurrentPxf(f);
	    } else if (f.isFile()) {
		int l = file_list[i].length();
		if (l >= 4  &&  file_list[i].substring(l-4).equalsIgnoreCase(".pxf")) {
		    if (output_midway_progress)
			System.err.println("Reading " + file_list[i]);

		    FileInputStream fis = new FileInputStream(f);
		    DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
		    PxfFile pxf = new PxfFile();
		    pxf.read(dis);
		    dis.close();

		    Vector all_list = pxf.pair_list.getAllList();
		    for (int j = 0 ; j < all_list.size () ; j++) {
			Vector list = (Vector)all_list.elementAt(j);
			for (int k = 0 ; k < list.size () ; k++) {
			    PairData pair = (PairData)list.elementAt(k);
			    if (pair.getType() == PairType.ND) {
				if (! pair.hasReference())
				    dc_table.add(pair, pxf.image_info);
			    }
			}
		    }

		    info_list.addElement(pxf.image_info);
		}
	    }
	}
    }

    private void checkMagnitudeDatabase ( ) throws Exception {
	if (output_midway_progress) {
	    PrintManager print_manager = new PrintManager();
	    print_manager.outputToConsole(false);
	    print_manager.outputToErr(true);
	    dc_table.setPrintManager(print_manager);
	}
	dc_table.checkMagnitudeDatabase();
    }

    private void checkPastPxf ( ) throws Exception {
	ImageInformationList image_info_list = new ImageInformationList();

	File file = new File(MisaoProperty.getImageInformationListPath());
	FileInputStream fis = new FileInputStream(file);
	DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
	image_info_list.read(dis);
	dis.close();

	for (int i = 0 ; i < image_info_list.size() ; i++) {
	    ImageInformation image_info = image_info_list.elementAt(i);

	    if (output_midway_progress)
		System.err.println(image_info.image_url);

	    dc_table.checkPastPxf(image_info);
	}
    }

    public void print ( PrintStream ps ) {
	ps.println("**********************************************************************");
	ps.println("Candidates of new objects");
	ps.println("**********************************************************************");
	Vector list = dc_table.getNewObjectList();
	for (int i = 0 ; i < list.size() ; i++) {
	    ps.println("======================================================================");
	    MagnitudeDatabaseStar mag_star = (MagnitudeDatabaseStar)list.elementAt(i);
	    ps.println(mag_star.getName());
	    mag_star.print(ps);
	}
	ps.println("");

	ps.println("**********************************************************************");
	ps.println("Candidates of variable stars");
	ps.println("**********************************************************************");
	list = dc_table.getVariableList();
	for (int i = 0 ; i < list.size() ; i++) {
	    ps.println("======================================================================");
	    MagnitudeDatabaseStar mag_star = (MagnitudeDatabaseStar)list.elementAt(i);
	    ps.println(mag_star.getName());
	    mag_star.print(ps);
	}
	ps.println("");

	ps.println("**********************************************************************");
	ps.println("Normal stars");
	ps.println("**********************************************************************");
	list = dc_table.getNormalList();
	for (int i = 0 ; i < list.size() ; i++) {
	    ps.println("======================================================================");
	    MagnitudeDatabaseStar mag_star = (MagnitudeDatabaseStar)list.elementAt(i);
	    ps.println(mag_star.getName());
	    mag_star.print(ps);
	}
	ps.println("");

	ps.println("**********************************************************************");
	ps.println("No data in past");
	ps.println("**********************************************************************");
	list = dc_table.getNoPastList();
	for (int i = 0 ; i < list.size() ; i++) {
	    ps.println("======================================================================");
	    MagnitudeDatabaseStar mag_star = (MagnitudeDatabaseStar)list.elementAt(i);
	    ps.println(mag_star.getName());
	    mag_star.print(ps);
	}
	ps.println("");
    }
}
