/*
 * DbvsData.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.host;

import misao.pixy.*;
import misao.util.*;

public class DbvsData {
    private String real_brightest_mag;
    private String brightest_mag;
    private String faintest_mag;
    private double real_b_mag;
    private double b_mag;
    private double f_mag;
    private String start_date;
    private String end_date;
    private String start_format_date;
    private String end_format_date;
    private int obs;

    public DbvsData ( ) {
	real_brightest_mag = "";
	brightest_mag = "";
	faintest_mag = "";
	real_b_mag = 0.0;
	f_mag = 0.0;
	start_date = "";
	end_date = "";
	start_format_date = "";
	end_format_date = "";
	obs = 0;
    }

    public String getBrightestMag ( ) {
	return brightest_mag;
    }

    public String getFaintestMag ( ) {
	return faintest_mag;
    }

    public int getObsCount ( ) {
	return obs;
    }

    public String getStartDate ( ) {
	return start_date;
    }

    public String getEndDate ( ) {
	return end_date;
    }

    public void update ( String date, String mag ) {
	obs++;

	String format_date = formatDate(date);
	if (format_date != null) {
	    if (start_date.length() == 0) {
		start_date = date;
		end_date = date;
		start_format_date = format_date;
		end_format_date = format_date;
	    } else {
		end_date = date;
		end_format_date = format_date;
	    }
	}

	mag = formatMag(mag);
	if (mag != null) {
	    if (brightest_mag.length() == 0) {
		brightest_mag = mag;
		faintest_mag = mag;
		b_mag = doubleValueMag(mag);
		f_mag = b_mag;
	    } else {
		double m = doubleValueMag(mag);
		if (m < b_mag) {
		    brightest_mag = mag;
		    b_mag = m;
		}
		if (m > f_mag) {
		    faintest_mag = mag;
		    f_mag = m;
		}
	    }

	    if (! (mag.charAt(0) == '<'  ||  mag.charAt(0) == '>'  ||  mag.charAt(0) == '['  ||  mag.charAt(0) == ']'  ||  mag.charAt(0) == '('  ||  mag.charAt(0) == ')')) {
		double m = doubleValueMag(mag);
		if (real_brightest_mag.length() == 0  ||  m < real_b_mag) {
		    real_brightest_mag = mag;
		    real_b_mag = m;

		    brightest_mag = mag;
		    b_mag = m;
		}
	    }
	}
    }

    private String formatDate ( String date ) {
	if (date.length() < 8)
	    return null;

	String month = date.substring(4, 6);
	if (month.charAt(0) == '0')
	    month = date.substring(5, 6);
	String day = date.substring(6, 8);
	if (day.charAt(0) == '0')
	    day = date.substring(7, 8);
	date = date.substring(0, 4) + " " + JulianDay.getMonthCode(Integer.parseInt(month)) + " " + day;
	return date;
    }

    private String formatMag ( String mag ) {
	int p = mag.indexOf('.');
	if (p < 0) {
	    String s = "";
	    for (int i = 0 ; i < mag.length() ; i++) {
		if ('0' <= mag.charAt(i)  &&  mag.charAt(i) <= '9')
		    s += mag.charAt(i);
	    }
	    int count = 0;
	    if (Integer.parseInt(s) >= 300)
		count = 1;
	    for (int i = mag.length() - 1 ; i >= 0 ; i--) {
		if ('0' <= mag.charAt(i)  &&  mag.charAt(i) <= '9') {
		    if (count == 0) {
			p = i;
			break;
		    }

		    count--;
		}
	    }
	    mag = mag.substring(0, p) + '.' + mag.substring(p);
	}
	return mag;
    }

    public static double doubleValueMag ( String s ) {
	int start = -1;
	for (int i = 0 ; i < s.length() ; i++) {
	    if ('0' <= s.charAt(i)  &&  s.charAt(i) <= '9') {
		start = i;
		break;
	    }
	}
	if (start < 0)
	    return 0.0;

	if (start >= 1  &&  s.charAt(start - 1) == '-')
	    start--;

	int end = 0;
	for (end = start + 1 ; end < s.length() ; end++) {
	    if (! (('0' <= s.charAt(end)  &&  s.charAt(end) <= '9')  ||  s.charAt(end) == '.'))
		break;
	}

	return Double.valueOf(s.substring(start, end)).doubleValue();
    }

    public String toString ( ) {
	return brightest_mag + " - " + faintest_mag + " mag by " + obs + " obs (" + start_format_date + " - " + end_format_date + ")";
    }
}
