/*
 * ChartFrame.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import misao.util.*;

public class ChartFrame extends Frame implements Runnable, EventHandler {
    private Thread thread;

    protected StarList star_list;
    protected Size size;
    protected MapFunction mf;
    protected Position pos;
    protected double faintest_mag;

    protected CatalogCanvas catalog_canvas;
    protected Checkbox checkBgBlack;
    protected Button btnView;

    // dummy
    protected ChartFrame ( ) {
    }

    public ChartFrame ( String title, StarList list, Size s, MapFunction f, Position p, double mag ) {
	star_list = list;
	size = s;
	mf = f;
	pos = p;
	faintest_mag = mag;

	init();

	setTitle("PIXY system - " + title);
	setSize(size.getWidth() + 10, size.getHeight() + 40 + 25);
	setVisible(true);
    }

    public void init ( ) {
	checkBgBlack = new Checkbox("Black Background");
	checkBgBlack.setState(false);
	btnView = new Button("View");

	Panel panel = new Panel(new FlowLayout());
	panel.add(checkBgBlack);
	panel.add(btnView);

	catalog_canvas = new CatalogCanvas(this, star_list, mf, size, pos, faintest_mag);

	setLayout(new BorderLayout());
	add("North",  panel);
	add("Center", catalog_canvas);
    }

    public void start ( ) {
	thread = new Thread(this);
	thread.start();
    }

    public void stop ( ) {
	if (thread != null) {
	    thread.stop();
	    thread = null;
	}
    }

    public void mark ( Point pt ) {
	catalog_canvas.setMark(pt);
	catalog_canvas.repaint();
    }

    public boolean action (Event ev, Object obj) {
	if (ev.target instanceof Button) {
	    if (btnView.getLabel().equals(obj)) {
		catalog_canvas.bgBlack(checkBgBlack.getState());
		catalog_canvas.updateOffScreen();
		catalog_canvas.repaint();

		return true;
	    }
	}

	return false;
    }

    public void run ( ) {
	while (true) {
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
		System.err.println("sleep() is interrupted. " + e.getMessage());
		break;
	    }
	}
    }
}
