/*
 * ImageFrame.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import misao.util.*;

public class ImageFrame extends Frame implements Runnable, EventHandler {
    private Thread thread;

    private ImagePixy image;

    private ImageCanvas image_canvas;
    private TextField text_min;
    private TextField text_max;
    private Button btnView;

    public ImageFrame ( ImagePixy i ) throws Exception {
	image = i;

	init();

	setTitle("PIXY system - Image");
	setSize(image.getWidth() + 10, image.getHeight() + 40 + 25);
	setVisible(true);
    }

    public void init ( ) throws Exception {
	MinMaxAverage mma = new ImageFileFits(image).getBestMinMax();

	String s = String.valueOf(mma.getMin());
	if (mma.getMax() - mma.getMin() > 10) {
	    text_min = new TextField(String.valueOf((int)mma.getMin()), 10);
	    text_max = new TextField(String.valueOf((int)mma.getMax()), 10);
	} else {
	    text_min = new TextField(String.valueOf(mma.getMin()), 10);
	    text_max = new TextField(String.valueOf(mma.getMax()), 10);
	}
	btnView = new Button("View");

	Panel panel = new Panel(new FlowLayout());
	panel.add(text_min);
	panel.add(text_max);
	panel.add(btnView);

	image_canvas = new ImageCanvas(image.getImage(mma.getMin(), mma.getMax()), image.getSize());

	setLayout(new BorderLayout());
	add("North",  panel);
	add("Center", image_canvas);
    }

    public void start ( ) {
	thread = new Thread(this);
	thread.start();
    }

    public void stop ( ) {
	if (thread != null) {
	    thread.stop();
	    thread = null;
	}
    }

    public void mark ( Point pt ) {
	image_canvas.setMark(pt);
	image_canvas.repaint();
    }

    public boolean action (Event ev, Object obj) {
	if (ev.target instanceof Button) {
	    if (btnView.getLabel().equals(obj)) {
		try {
		    double min = (double)(Float.valueOf(text_min.getText().trim()).floatValue());
		    double max = (double)(Float.valueOf(text_max.getText().trim()).floatValue());
		    image_canvas.setImage(image.getImage(min, max));
		    image_canvas.repaint();

		    return true;
		} catch ( NumberFormatException e ) {
		    System.err.println("Number format exception: " + text_min.getText() + " " + text_max.getText());
		} catch ( Exception e ) {
		}
	    }
	}

	return false;
    }

    public void run ( ) {
	while (true) {
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
		System.err.println("sleep() is interrupted. " + e.getMessage());
		break;
	    }
	}
    }
}
