/*
 * ImageInformationList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.util.*;
import misao.util.*;

public class ImageInformationList implements Printable {
    private Vector list;

    public ImageInformationList ( ) {
	list = new Vector();
    }

    public int size ( ) {
	return list.size();
    }

    public ImageInformation elementAt ( int i ) {
	return (ImageInformation)list.elementAt(i);
    }

    public void read ( DataInputStream dis ) throws Exception {
	String record = "";
	int line_count = 0;
	try {
	    record = dis.readLine();
	    line_count++;
	    while (record != null) {
		if (record.equals("[Image Information]")) {
		    ImageInformation image_info = new ImageInformation();

		    record = dis.readLine();
		    line_count++;
		    while (record != null  &&  record.length() > 0) {
			image_info.setParameter(new Parameter(record));

			record = dis.readLine();
			line_count++;
		    }

		    add(image_info);
		}

		if (record != null) {
		    record = dis.readLine();
		    line_count++;
		}
	    }
	} catch (Exception e) {
	    System.err.println("" + line_count + ": " + record);
	    throw e;
	}
    }

    public ImageInformation readNextImageInformation ( DataInputStream dis ) throws Exception {
	String record = "";
	try {
	    record = dis.readLine();
	    while (record != null) {
		if (record.equals("[Image Information]")) {
		    ImageInformation image_info = new ImageInformation();

		    record = dis.readLine();
		    while (record != null  &&  record.length() > 0) {
			image_info.setParameter(new Parameter(record));
			record = dis.readLine();
		    }

		    return image_info;
		}

		if (record != null)
		    record = dis.readLine();
	    }
	} catch (Exception e) {
	    System.err.println(record);
	    throw e;
	}

	return null;
    }

    public void add ( ImageInformation image_info ) {
	for (int i = list.size() ; i >= 1 ; i--) {
	    ImageInformation info = (ImageInformation)list.elementAt(i-1);
	    if (info.image_url.equals(image_info.image_url)) {
		list.removeElementAt(i-1);
		list.insertElementAt(image_info, i-1);
		return;
	    } else if (info.exposure_start.getJD() <= image_info.exposure_start.getJD()) {
		if (i == list.size())
		    list.addElement(image_info);
		else
		    list.insertElementAt(image_info, i);
		return;
	    }
	}
	if (list.size() == 0)
	    list.addElement(image_info);
	else
	    list.insertElementAt(image_info, 0);
    }

    public void print ( PrintStream ps ) {
	for (int i = 0 ; i < list.size() ; i++) {
	    ImageInformation image_info = (ImageInformation)list.elementAt(i);
	    ps.println("[Image Information]");
	    image_info.print(ps);
	    ps.println("");
	}
    }
}
