/*
 * MapFunctionCandidate.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import misao.util.*;

public class MapFunctionCandidate extends MapFunction {
    private static final double d_rate = 1.025;
    private static final double d_theta = 2.0;

    public MapFunctionCandidate ( double R, double T, Position Pos ) {
	super(R, T, Pos);
    }

    // map function: tri1 -> tri2
    public static MapFunctionCandidate calculate ( TriStars tri1, TriStars tri2 ) {
	SimultaneousEquation sex = new SimultaneousEquation(3);
	SimultaneousEquation sey = new SimultaneousEquation(3);

	for (int i = 0 ; i < 3 ; i++) {
	    sex.coef[i][0] += tri1.element(i).getX();
	    sex.coef[i][1] -= tri1.element(i).getY();
	    sex.coef[i][2] += 1.0;
	    sex.coef[i][3] -= tri2.element(i).getX();

	    sey.coef[i][0] += tri1.element(i).getX();
	    sey.coef[i][1] += tri1.element(i).getY();
	    sey.coef[i][2] += 1.0;
	    sey.coef[i][3] -= tri2.element(i).getY();
	}
	sex.solve();
	sey.solve();

	double r_cost = sex.getAnswer(0);
	double r_sint = sex.getAnswer(1);
	double x0 = sex.getAnswer(2);
	double r1 = Math.sqrt(r_cost * r_cost + r_sint * r_sint);
	double theta1 = Math.atan2(r_sint, r_cost) / Astro.RAD;

	r_sint = sey.getAnswer(0);
	r_cost = sey.getAnswer(1);
	double y0 = sey.getAnswer(2);
	double r2 = Math.sqrt(r_cost * r_cost + r_sint * r_sint);
	double theta2 = Math.atan2(r_sint, r_cost) / Astro.RAD;

	MapFunctionCandidate mf = null;
	if (1.0 / d_rate < r1 / r2  &&  r1 / r2 < d_rate  &&  
	    Astro.angleDistance(theta1, theta2) < d_theta) {
	    mf = new MapFunctionCandidate(Math.sqrt(r1 * r2), Astro.angleMean(theta1, theta2), new Position(x0, y0));
	}

	return mf;
    }

    public double checkValidity ( StarList list1, StarList list2, Size image_size, int check_count ) {
	StarList l2 = new StarList(list2, (int)((double)list2.size() / getRate()));
	StarListIndex index2 = new StarListIndex(l2, new Position(- (double)image_size.getWidth() / 2.0, - (double)image_size.getHeight() / 2.0), new Position((double)image_size.getWidth() / 2.0, (double)image_size.getHeight() / 2.0), 5, inverse());
	index2.removeOutOfArea(inverse());

	int total_count = 0;
	int match_count = 0;
	for (int i = 0 ; i < check_count ; i++) {
	    if (i < list1.size()) {
		StarData sd = list1.elementAt(i);
		StarList l = index2.getFirstList(sd.getPosition(), new misao.util.MapFunction());
		total_count++;
		int c = 0;
		while (l != null) {
		    c += l.size();
		    l = index2.getNextList();
		}
		if (c > 0)
		    match_count++;
	    }
	}

	return (double)match_count / (double)total_count;
    }
}
