/*
 * PixyInputData.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.net.*;
import java.util.*;
import misao.util.*;

public class PixyInputData implements Printable {
    public String image_url_string;
    public URL image_url;
    public JulianDay exposure_start;
    public double exposure_seconds;
    public Catalog catalog;
    public String[] root_dir;
    public Coor center_coor;
    public double position_error;
    public double fov;
    public String mag_code;
    public double limiting_mag;
    public boolean east_right;
    public boolean sbig_ratio;
    public boolean reject_cosmic_ray;
    public boolean detail_star_data;

    public String output_filename;
    public String pxf_filename;

    public PixyInputData ( ) {
	image_url_string = "";
	exposure_start = new JulianDay();
	exposure_seconds = 0.0;
	catalog = null;
	center_coor = new Coor();
	position_error = 0.0;
	fov = 1.0;
	limiting_mag = Astro.ILLEGAL_MAG;
	mag_code = "";
	east_right = false;
	sbig_ratio = false;
	reject_cosmic_ray = false;
	detail_star_data = false;

	output_filename = "";
	pxf_filename = "";
    }

    public void setParameter ( Parameter p ) throws Exception {
	if (p.getKey().equals("image")) {
	    image_url_string = p.getValue();
	    image_url = new URL(new URLString(image_url_string).toString());
	} else if (p.getKey().equals("exposure_start")) {
	    exposure_start = JulianDay.createJulianDay(p.getValue());
	} else if (p.getKey().equals("exposure")) {
	    exposure_seconds = Astro.doubleSecondValueOf(p.getValue());
	} else if (p.getKey().equals("catalog")) {
	    catalog = CatalogIndex.getCatalogOfCode(p.getValue());
	    if (catalog == null)
		catalog = CatalogIndex.getCatalogOfName(p.getValue());
	} else if (p.getKey().equals("catalog_path")) {
	    StringTokenizer st = new StringTokenizer(p.getValue());
	    root_dir = new String[st.countTokens()];
	    int i = 0;
	    while (st.hasMoreTokens())
		root_dir[i++] = st.nextToken();
	} else if (p.getKey().equals("center")) {
	    center_coor = Coor.createCoor(p.getValue());
	} else if (p.getKey().equals("pos_error")) {
	    position_error = Astro.doubleDegreeValueOf(p.getValue());
	} else if (p.getKey().equals("field_of_view")) {
	    fov = Astro.doubleDegreeValueOf(p.getValue());
	} else if (p.getKey().equals("limiting_mag")) {
	    limiting_mag = Astro.doubleDegreeValueOf(p.getValue());
	} else if (p.getKey().equals("mag_code")) {
	    mag_code = p.getValue();
	} else if (p.getKey().equals("mirror")) {
	    if (p.getValue().length() > 0)
		east_right = (p.getValue().charAt(0) == 'y'  ||  p.getValue().charAt(0) == 'Y');
	} else if (p.getKey().equals("sbig_ratio")) {
	    if (p.getValue().length() > 0)
		sbig_ratio = (p.getValue().charAt(0) == 'y'  ||  p.getValue().charAt(0) == 'Y');
	} else if (p.getKey().equals("reject_CR")) {
	    if (p.getValue().length() > 0)
		reject_cosmic_ray = (p.getValue().charAt(0) == 'y'  ||  p.getValue().charAt(0) == 'Y');
	} else if (p.getKey().equals("detail")) {
	    if (p.getValue().length() > 0)
		detail_star_data = (p.getValue().charAt(0) == 'y'  ||  p.getValue().charAt(0) == 'Y');
	} else if (p.getKey().equals("log")) {
	    output_filename = p.getValue();
	} else if (p.getKey().equals("pxf")) {
	    pxf_filename = p.getValue();
	}
    }

    public void print ( PrintStream ps ) {
	new Parameter("image", image_url_string).print(ps);
	new Parameter("exposure_start", exposure_start.toString()).print(ps);
	new Parameter("exposure", "" + exposure_seconds + " sec").print(ps);
	new Parameter("catalog", catalog.getName()).print(ps);
	new Parameter("center", center_coor.toStringRA() + " " + center_coor.toStringDecl()).print(ps);
	new Parameter("pos_error", "" + position_error + " deg").print(ps);
	new Parameter("field_of_view", "" + fov + " deg").print(ps);
	if (Math.abs(limiting_mag - Astro.ILLEGAL_MAG) > 1.0)
	    new Parameter("limiting_mag", "" + limiting_mag + " mag").print(ps);
	if (mag_code.length() > 0)
	    new Parameter("mag_code", mag_code).print(ps);
	new Parameter("mirror", east_right).print(ps);
	new Parameter("sbig_ratio", sbig_ratio).print(ps);
	new Parameter("reject_CR", reject_cosmic_ray).print(ps);
    }
}
