/*
 * PsfDistribution.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import java.util.*;
import misao.util.*;

public class PsfDistribution {
    private Size size;
    private DetectData sd1[][];
    private DetectData sd2[][];
    private DetectData sd3[][];
    private ImageGray power1;
    private ImageGray power2;
    private ImageGray power3;

    public PsfDistribution ( Size s ) throws Exception {
	size = s;

	sd1 = new DetectData[size.getWidth()][size.getHeight()];
	sd2 = new DetectData[size.getWidth()][size.getHeight()];
	sd3 = new DetectData[size.getWidth()][size.getHeight()];

	power1 = new ImageGray(size);
	power2 = new ImageGray(size);
	power3 = new ImageGray(size);

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < size.getWidth() ; pt.x++) {
		sd1[pt.x][pt.y] = null;
		sd2[pt.x][pt.y] = null;
		sd3[pt.x][pt.y] = null;
		power1.setValue(pt, 0.0);
		power2.setValue(pt, 0.0);
		power3.setValue(pt, 0.0);
	    }
	}
    }

    public DetectData getStarData ( Point pt, int i ) {
	switch ( i ) {
	    case 0:
		return sd1[pt.x][pt.y];
	    case 1:
		return sd2[pt.x][pt.y];
	    case 2:
		return sd3[pt.x][pt.y];
	}
	return null;
    }

    public double getPower ( Point pt, int i ) {
	switch ( i ) {
	    case 0:
		return power1.getValue(pt);
	    case 1:
		return power2.getValue(pt);
	    case 2:
		return power3.getValue(pt);
	}
	return 0.0;
    }

    public double getTotalPower ( Point pt ) {
	return power1.getValue(pt) + power2.getValue(pt) + power3.getValue(pt);
    }

    public int[] getPixelCount ( DetectData sd ) {
	int[] count_array = new int[3];
	for (int i = 0 ; i < 3 ; i++)
	    count_array[i] = 0;

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < size.getWidth() ; pt.x++) {
		if (sd1[pt.x][pt.y] == sd)
		    count_array[0]++;
		if (sd2[pt.x][pt.y] == sd)
		    count_array[1]++;
		if (sd3[pt.x][pt.y] == sd)
		    count_array[2]++;
	    }
	}

	return count_array;
    }

    public Vector getPixelPoint ( DetectData sd ) {
	Vector list = new Vector();

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < size.getWidth() ; pt.x++) {
		if (sd1[pt.x][pt.y] == sd  ||  sd2[pt.x][pt.y] == sd  ||  sd3[pt.x][pt.y] == sd) {
		    list.addElement(new Point(pt.x, pt.y));
		}
	    }
	}

	return list;
    }

    // determine three owner stars for each pixel
    public void owner ( DetectList detect_list, double aperture_radius ) {
	for (int i = 0 ; i < detect_list.size() ; i++) {
	    DetectData sd = (DetectData)detect_list.elementAt(i);

	    Point pt = new Point(0, 0);
	    for (pt.y = (int)(sd.getY() - aperture_radius) ; pt.y <= (int)(sd.getY() + aperture_radius) ; pt.y++) {
		for (pt.x = (int)(sd.getX() - aperture_radius) ; pt.x <= (int)(sd.getX() + aperture_radius) ; pt.x++) {
		    if (0 <= pt.x  &&  pt.x < size.getWidth()  &&  0 <= pt.y  &&  pt.y < size.getHeight()) {
			// influence to the pixel
			double p = sd.getPSF().power(new Position((double)pt.x, (double)pt.y));

			if (p >= 1.0) {
			    if (p > power3.getValue(pt)) {
				sd3[pt.x][pt.y] = sd;
				power3.setValue(pt, p);
				if (p > power2.getValue(pt)) {
				    sd3[pt.x][pt.y] = sd2[pt.x][pt.y];
				    power3.setValue(pt, power2.getValue(pt));
				    sd2[pt.x][pt.y] = sd;
				    power2.setValue(pt, p);
				    if (p > power1.getValue(pt)) {
					sd2[pt.x][pt.y] = sd1[pt.x][pt.y];
					power2.setValue(pt, power1.getValue(pt));
					sd1[pt.x][pt.y] = sd;
					power1.setValue(pt, p);
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }
}
