/*
 * PxfFile.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import misao.util.*;

public class PxfFile implements Printable {
    public PixyInputData input_data;
    public ImageInformation image_info;
    public PairList pair_list;

    public PxfFile ( ) {
	input_data = null;
	image_info = null;
	pair_list = null;
    }

    public PxfFile ( PixyInputData _input_data, ImageInformation _image_info, PairList _pair_list ) {
	this();

	input_data = _input_data;
	image_info = _image_info;
	pair_list = _pair_list;
    }

    public static String getFilenameBody ( String pxf_filename ) {
	String s = pxf_filename.toLowerCase();
	if (s.length() > 4  &&  s.substring(s.length() - 4).equals(".pxf"))
	    return pxf_filename.substring(0, s.length() - 4);
	return pxf_filename;
    }

    public static PxfFile readPxfFileOfImage ( String image_filename ) {
	for (int i = 0 ; i < 4 ; i++) {
	    String pxf_filename = "";
	    int ptr = image_filename.lastIndexOf('.');

	    switch (i) {
	    case 0:
		pxf_filename = image_filename + ".pxf";
		break;
	    case 1:
		pxf_filename = image_filename + ".PXF";
		break;
	    case 2:
		pxf_filename = image_filename.substring(0,ptr) + ".pxf";
		break;
	    case 3:
		pxf_filename = image_filename.substring(0,ptr) + ".PXF";
		break;
	    }

	    try {
		File file = new File(pxf_filename);
		FileInputStream fis = new FileInputStream(file);
		DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
		PxfFile pxf = new PxfFile();
		pxf.read(dis);
		dis.close();

		return pxf;
	    } catch (Exception e) {
	    }
	}

	return null;
    }

    public void print ( PrintStream ps ) {
	if (input_data != null) {
	    ps.println("[Input Data]");
	    input_data.print(ps);
	    ps.println("");
	}

	if (image_info != null) {
	    ps.println("[Image Information]");
	    image_info.print(ps);
	    ps.println("");
	}

	if (pair_list != null) {
	    if (pair_list.isOutputStatistics()) {
		ps.println("[Statistics]");
		pair_list.printStatistics(ps);
		ps.println("");
	    }
	    if (pair_list.isOutputAllData()) {
		ps.println("[Data]");
		pair_list.printData(ps);
		ps.println("");
	    }
	}
    }

    public void read ( DataInputStream dis ) throws Exception {
	int PS_count = 0;
	int RM_count = 0;
	int RP_count = 0;
	int NP1_count = 0;
	int NP2_count = 0;
	int NR_count = 0;

	boolean pixy_flag = false;

	int line_count = 0;
	String record = "";
	try {
	    record = dis.readLine();
	    line_count++;
	    while (record != null) {
		if (record.equals("[Input Data]")) {
		    input_data = new PixyInputData();

		    record = dis.readLine();
		    line_count++;
		    while (record != null  &&  record.length() > 0) {
			input_data.setParameter(new Parameter(record));

			record = dis.readLine();
			line_count++;
		    }
		} else if (record.equals("[Image Information]")) {
		    image_info = new ImageInformation();

		    record = dis.readLine();
		    line_count++;
		    while (record != null  &&  record.length() > 0) {
			image_info.setParameter(new Parameter(record));

			record = dis.readLine();
			line_count++;
		    }
		} else if (record.equals("[Statistics]")) {
		    record = dis.readLine();
		    line_count++;
		    while (record != null  &&  record.length() > 0) {
			Parameter p = new Parameter(record);
			if (p.getKey().equals("PS")) {
			    PS_count = Integer.parseInt(p.getValue());
			} else if (p.getKey().equals("RM")) {
			    RM_count = Integer.parseInt(p.getValue());
			} else if (p.getKey().equals("RP")) {
			    RP_count = Integer.parseInt(p.getValue());
			} else if (p.getKey().equals("NP1")) {
			    NP1_count = Integer.parseInt(p.getValue());
			} else if (p.getKey().equals("NP2")) {
			    NP2_count = Integer.parseInt(p.getValue());
			} else if (p.getKey().equals("ND")) {
			    NP1_count = Integer.parseInt(p.getValue());
			    pixy_flag = true;
			} else if (p.getKey().equals("NC")) {
			    NP2_count = Integer.parseInt(p.getValue());
			    pixy_flag = true;
			} else if (p.getKey().equals("NR")) {
			    NR_count = Integer.parseInt(p.getValue());
			}

			record = dis.readLine();
			line_count++;
		    }
		} else if (record.equals("[Data]")) {
		    pair_list = new PairList();
		    if (pixy_flag)
			pair_list = new PixyPairList();

		    pair_list.read(dis);

		    // for compatibility
		    PairData pair = pair_list.elementPairAt(0);
		    DetectData sd_detect = (DetectData)pair.getFirstData();
		    image_info.mag_code = sd_detect.getMagCode();
		}

		if (record != null) {
		    record = dis.readLine();
		    line_count++;
		}
	    }

	    if (pixy_flag)
		((PixyPairList)pair_list).selectAtEdge(image_info.image_size, new Position(0,0));
	} catch (Exception e) {
	    System.err.println("" + line_count + ": " + record);
	    throw e;
	}
    }
}
