/*
 * TriGraph.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.util.Vector;
import misao.util.*;

public class TriGraph {
    private StarList list;
    private Graph graph;

    private int ptr;

    public TriGraph ( StarList l ) {
	list = l;
	graph = list.getPositionList().createMinimumGlobalTree();
	ptr = 0;
    }

    public int getNodeCount ( ) {
	return list.size();
    }

    public StarData getNodeAt ( int i) {
	return (StarData)list.elementAt(i);
    }

    public StarData getFirstNode ( ) {
	ptr = 0;
	return getNextNode();
    }

    public StarData getNextNode ( ) {
	if (ptr >= list.size())
	    return null;

	ptr++;
	return (StarData)list.elementAt(ptr-1);
    }

    public boolean isConnected ( int i, int j ) {
	return graph.isConnected(i, j);
    }

    public boolean isReachable ( int i, int j ) {
	return graph.isReachable(i, j);
    }

    public Vector createTriList ( ) {
	return createTriList((int)Math.sqrt(graph.getNodeCount()));
    }

    public Vector createTriList ( int steps ) {
	boolean[][] flag = new boolean[graph.getNodeCount()][graph.getNodeCount()];
	for (int i = 0 ; i < graph.getNodeCount() ; i++) {
	    for (int j = 0 ; j < graph.getNodeCount() ; j++)
		flag[i][j] = false;
	}
	for (int i = 0 ; i < graph.getNodeCount() ; i++) {
	    int[] index = graph.getIndexListReachableInSteps(i,steps);
	    for (int j = 0 ; j < index.length ; j++)
		flag[i][index[j]] = true;
	}

	Vector l = new Vector();
	for (int i1 = 0 ; i1 < graph.getNodeCount() - 2 ; i1++) {
	    for (int i2 = i1 +1 ; i2 < graph.getNodeCount() - 1 ; i2++) {
		if (flag[i1][i2]) {
		    for (int i3 = i2 + 1 ; i3 < graph.getNodeCount() ; i3++) {
			if (flag[i1][i3]) {
			    TriStars tri = new TriStars(list.elementAt(i1), list.elementAt(i2), list.elementAt(i3));
			    l.addElement(tri);
			}
		    }
		}
	    }
	}

	return l;
    }
}
