/*
 * Installer.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.ref;

import java.io.*;
import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.catalog.*;

public class Installer {
    public static void main ( String[] args ) throws Exception {
	System.out.println("Catalog:                     type  err(\")");

	DataInputStream din = new DataInputStream(System.in);

	System.out.println("[Variable stars]");
	System.out.println("1   GCVS2000                    V   5.0");
	System.out.println("2   NSV                         V   5.0");
	System.out.println("3   NSV Supplement              V   5.0");
	System.out.println("4   MisV                        V   5.0");
	System.out.println("5   TmzV                        V   5.0");
	System.out.println("6   HadV                        V   5.0");
	System.out.println("7   YmoV                        V   5.0");
	System.out.println("8   KnkV                        V   5.0");
	System.out.println("9   BnV                         V   5.0");
	System.out.println("10  nova.lst since 1995         V   5.0");
	System.out.println("11  supernova.lst since 1996    V   5.0");
	System.out.println("12  LD                          V   5.0(15.0)");
	System.out.println("13  OGLE-LT                     V   5.0");
	System.out.println("14  OGLE-BUL                    V   1.5");
	System.out.println("15  FASTT-1                     V   5.0");
	System.out.println("16  TASS                        V   5.0");
	System.out.println("17  HassfortherV                V   5.0");
	System.out.println("18  ROTSE-I                     V   5.0");
	System.out.println("19  BeV                         V   5.0");
	System.out.println("20  ROSAT CVs                   V   5.0");
//	System.out.println("49  MisaoSpecial                V   5.0");

	System.out.println("[Revision of variable stars]");
	System.out.println("51  GCVS_REV                    V   5.0");
	System.out.println("52  GCVS_REV (iii.dat)          V   5.0");
	System.out.println("53  GCVS_REV by Williams        V   5.0");
	System.out.println("54  MisaoRev                    V   5.0");
	System.out.println("55  MisaoConfirm                V   5.0");

	System.out.println("[Suspicious variable objects]");
	System.out.println("101 IRAS PSC                    S  30.0");
	System.out.println("102 IRAS FSC                    S  30.0");
	System.out.println("103 Veron                       S   5.0");
	System.out.println("104 Markarian                   S  90.0");

	System.out.println("[Catalog errors]");
	System.out.println("201 STM                         E   1.5");
	System.out.println("202 MisE.GSC                    E   1.5");
	System.out.println("203 MisE.UA1                    E   1.5");

	System.out.println("[Nebulae and clusters]");
	System.out.println("301 NGC2000                     N  90.0");
	System.out.println("302 MCG                         N  90.0");
	System.out.println("303 MisF                        N   1.5");

	System.out.print("Input a number: ");
	int number = Integer.parseInt(din.readLine());

	System.out.print("Filename of catalog: ");
	String path = din.readLine();

	System.out.println("Type: ");
	ReferenceType.printMessage(System.out);
	System.out.print("Type: ");
	ReferenceType type = ReferenceType.getType(din.readLine().charAt(0));

	System.out.print("Positional error (arcsec): ");
	double pos_error = Double.valueOf(din.readLine()).doubleValue();

	System.out.print("Replace old data? [y/N]: ");
	String replace_flag_str = din.readLine();
	boolean replace_flag = false;
	if (replace_flag_str.equalsIgnoreCase("y"))
	    replace_flag = true;

	MisaoProperty.createReferenceDirectory();

	ReferenceCatalogBaseManager ref_catalog = new ReferenceCatalogBaseManager();

	ReferenceCatalog ref = null;
	switch (number) {
	case 1:   ref = new Gcvs();  break;
	case 2:   ref = new Nsv();  break;
	case 3:   ref = new NsvSupplement();  break;
	case 4:   ref = new MisV();  break;
	case 5:   ref = new TmzV();  break;
	case 6:   ref = new HadV();  break;
	case 7:   ref = new HadV("Minoru Yamamoto");  break;	// Yamamoto
	case 8:   ref = new HadV();  break;	// KnkV
	case 9:   ref = new BnV();  break;
	case 10:  ref = new NovaList();  break;
	case 11:  ref = new SupernovaList();  break;
	case 12:  ref = new LD();  break;
	case 13:  ref = new OgleLt();  break;
	case 14:  ref = new OgleBul();  break;
	case 15:  ref = new Fastt();  break;
	case 16:  ref = new Tass();  break;
	case 17:  ref = new Hassforther();  break;
	case 18:  ref = new Rotse();  break;
	case 19:  ref = new BeV();  break;
	case 20:  ref = new Rosat();  break;
	case 49:  ref = new MisaoRev(false, true);  break;	// MisaoSpecial

	case 51:  ref = new GcvsPatch();  break;
	case 52:  ref = new GcvsPatchIii();  break;
	case 53:  ref = new GcvsPatchWilliams();  break;
	case 54:  ref = new MisaoRev();  break;
	case 55:  ref = new MisaoRev(true, true);  break;	// MisaoConfirm

	case 101: ref = new IrasPsc();  break;
	case 102: ref = new IrasFsc();  break;
	case 103: ref = new Veron();  break;
	case 104: ref = new Mkn();  break;

	case 201: ref = new Stm();  break;
	case 202: ref = new MisEGsc();  break;
	case 203: ref = new MisEUa1();  break;

	case 301: ref = new NgcIc();  break;
	case 302: ref = new Mcg();  break;
	case 303: ref = new MisF();  break;
	}

	ref.setPath(path);

	ReferenceData ref_data = ref.readFirstData();
	while (ref_data != null) {
	    ref_data.setType(type);
	    ref_data.setPositionError(pos_error);

	    if (replace_flag) {
		ReferenceData old_data = ref_catalog.findData(ref_data);
		if (old_data == null) {
		    Vector l = ref_catalog.findData(ref_data.getCoor(), 1.0);
		    int j = 0;
		    for (j = 0 ; j < l.size() ; j++) {
			old_data = (ReferenceData)l.elementAt(j);
			if (old_data.getName().equals(ref_data.getName())) {
			    ref_catalog.removeData(old_data);
			    break;
			}
		    }
		    old_data = null;
		}
		if (old_data != null) {
		    if (ref instanceof GcvsPatch)
			ref_data = GcvsPatch.updateData(old_data, ref_data);
		    if (ref instanceof GcvsPatchWilliams)
			ref_data = GcvsPatchWilliams.updateData(old_data, ref_data);
		    if (ref instanceof GcvsPatchIii)
			ref_data = GcvsPatchIii.updateData(old_data, ref_data);
		    if (ref instanceof MisaoRev)
			ref_data = MisaoRev.updateData(old_data, ref_data);
		}

		ref_catalog.replaceData(ref_data);
		ref_data = ref.readNextData();
	    } else {
		ref_catalog.addData(ref_data);
		ref_data = ref.readNextData();
	    }
	}

	try {
	    ref_catalog.save();
	} catch (Exception e) {
	    System.err.println("Failed to save database.");
	}
    }
}
