/*
 * GcvsPatchIii.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
0010001 R     And *002123.0+381802 M           5.8      14.9     V 43135.           409.33           38   S3,5e-S8,8e(M7e) 00001 00002               002402.0+383438


0010002 S     And *003958.9+405940 SNI         5.8    < 16.      V 09775. (1885)                          (SNI)            V 377 V 338 =M31  V0894   004243.1+411606


 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class GcvsPatchIii extends ReferenceCatalog {
    public GcvsPatchIii ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.length() < 123)
	    return null;

	if (('0' <= record.charAt(19)  &&  record.charAt(19) <= '9')  &&
	    ('0' <= record.charAt(20)  &&  record.charAt(20) <= '9')  &&
	    ('0' <= record.charAt(21)  &&  record.charAt(21) <= '9')  &&
	    ('0' <= record.charAt(22)  &&  record.charAt(22) <= '9')  &&
	    ('0' <= record.charAt(23)  &&  record.charAt(23) <= '9')  &&
	    ('0' <= record.charAt(24)  &&  record.charAt(24) <= '9')  &&
	    ('0' <= record.charAt(28)  &&  record.charAt(28) <= '9')  &&
	    ('0' <= record.charAt(29)  &&  record.charAt(29) <= '9')  &&
	    ('0' <= record.charAt(30)  &&  record.charAt(30) <= '9')  &&
	    ('0' <= record.charAt(31)  &&  record.charAt(31) <= '9')  &&
	    ('0' <= record.charAt(32)  &&  record.charAt(32) <= '9')  &&
	    ('0' <= record.charAt(33)  &&  record.charAt(33) <= '9')) {

	    String code = record.substring(8,14).toUpperCase().trim();
	    String cons = record.substring(14,17).toUpperCase().trim();
	    String RA = record.substring(19,27);
	    String Decl = record.substring(27,34);
	    String type = record.substring(35,44).trim();
	    String mag_brightest = record.substring(44,54).trim();
	    String mag_faintest = record.substring(54,64).trim();
	    String mag_code = record.substring(64,66).trim();
	    String epoch = record.substring(67,80).trim();
	    String period = record.substring(80,99).trim();
	    String spectrum = record.substring(106,123).trim();

	    if (code.length() == 5  &&
		code.charAt(0) == 'V'  &&
		('0' <= code.charAt(1)  &&  code.charAt(1) <= '9')  &&
		('0' <= code.charAt(2)  &&  code.charAt(2) <= '9')  &&
		('0' <= code.charAt(3)  &&  code.charAt(3) <= '9')  &&
		('0' <= code.charAt(4)  &&  code.charAt(4) <= '9')) {
		int code_number = Integer.parseInt(code.substring(1));
		code = "V" + code_number;
	    }

	    String name = code + " " + cons;
	    String dbvs_name = cons + code;

	    String ra = RA;
	    if (ra.charAt(7) == ' ')
		ra = RA.substring(0,7) + "0";
	    if (ra.charAt(6) == ' ')
		ra = RA.substring(0,6) + "." + RA.substring(7,8);

	    CoorMapping coor = new CoorMapping(Coor.createCoor(ra + " " + Decl));
	    coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	    record = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl() + "  ";
	    record += mag_brightest + " - " + mag_faintest + " mag";
	    if (! mag_code.equals("-"))
		record += " (" + mag_code + ")";
	    if (! type.equals("-"))
		record += "  type:" + type;
	    if (! epoch.equals("-"))
		record += "  Epoch=" + epoch;
	    if (! period.equals("-"))
		record += "  Period=" + period;
	    if (! spectrum.equals("-"))
		record += "  Sp:" + spectrum;

	    ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

	    return data;
	}

	return null;
    }

    public static ReferenceData updateData ( ReferenceData orig_data, ReferenceData patch_data ) {
	String orig_record = orig_data.getRecord();
	int p = orig_record.indexOf("Decl.");
	p = orig_record.indexOf("  ", p);
	if (p < 0)
	    p = orig_record.indexOf("Decl.") + 17;

	String patch_record = patch_data.getRecord();
	int q = patch_record.indexOf("Decl.");
	q = patch_record.indexOf("  ", p);
	if (q < 0)
	    q = patch_record.indexOf("Decl.") + 17;

	String record = patch_record.substring(0,q) + orig_record.substring(p);
	ReferenceData data = new ReferenceData(orig_data.getName(), orig_data.getDbvsName(), patch_data.getCoor(), record, orig_data.getPositionError(), orig_data.getType());
	return data;
    }
}
