/*
 * Hassforther.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
No  (2000)                 GSC        IRAS         USNO-A2.0      TASS_ID
04  19 06 05.7  -01 29.98  -          19035-0134   0825-14421517  12051011

Comments:
No  Type     Magnitudes (TASS)  Comments
04  M        V=3D13.43 I=3D10.87    see IBVS 4742; P ca 250d
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class Hassforther extends ReferenceCatalog {
    private Vector data_list;

    private String last_record;

    public Hassforther ( ) {
	data_list = new Vector();

	last_record = "";
    }

    public ReferenceData createReferenceData ( String record ) {
	int p = record.indexOf("=E2=80=93");
	while (p >= 0) {
	    record = record.substring(0,p) + "-" + record.substring(p+9);
	    p = record.indexOf("=E2=80=93");
	}

	p = record.indexOf("=3D");
	while (p >= 0) {
	    record = record.substring(0,p) + "=" + record.substring(p+3);
	    p = record.indexOf("=3D");
	}

	if (record.length() > 0  &&  record.charAt(record.length() - 1) == '=') {
	    record = record.substring(0, record.length() - 1);
	    last_record += record;
	    return null;
	}

	record = last_record + record;
	last_record = "";

	if (record.length() > 31) {
	    if (('0' <= record.charAt(0)  &&  record.charAt(0) <= '9')  &&
		('0' <= record.charAt(1)  &&  record.charAt(1) <= '9')  &&
		record.charAt(2) == ' '  &&
		record.charAt(3) == ' '  &&
		('0' <= record.charAt(4)  &&  record.charAt(4) <= '9')  &&
		('0' <= record.charAt(5)  &&  record.charAt(5) <= '9')  &&
		record.charAt(6) == ' '  &&
		('0' <= record.charAt(7)  &&  record.charAt(7) <= '9')  &&
		('0' <= record.charAt(8)  &&  record.charAt(8) <= '9')  &&
		record.charAt(9) == ' '  &&
		('0' <= record.charAt(10)  &&  record.charAt(10) <= '9')  &&
		('0' <= record.charAt(11)  &&  record.charAt(11) <= '9')  &&
		record.charAt(12) == '.'  &&
		('0' <= record.charAt(13)  &&  record.charAt(13) <= '9')  &&
		record.charAt(14) == ' '  &&
		record.charAt(15) == ' '  &&
		(record.charAt(16) == '-'  ||  record.charAt(16) == '+')  &&
		('0' <= record.charAt(17)  &&  record.charAt(17) <= '9')  &&
		('0' <= record.charAt(18)  &&  record.charAt(18) <= '9')  &&
		record.charAt(19) == ' '  &&
		('0' <= record.charAt(20)  &&  record.charAt(20) <= '9')  &&
		('0' <= record.charAt(21)  &&  record.charAt(21) <= '9')  &&
		record.charAt(22) == '.'  &&
		('0' <= record.charAt(23)  &&  record.charAt(23) <= '9')  &&
		('0' <= record.charAt(24)  &&  record.charAt(24) <= '9')) {
		data_list.addElement(new String(record));
		return null;
	    }

	    if (('0' <= record.charAt(0)  &&  record.charAt(0) <= '9')  &&
		('0' <= record.charAt(1)  &&  record.charAt(1) <= '9')  &&
		record.charAt(2) == ' '  &&
		record.charAt(3) == ' ') {

		String data = "";
		for (int i = 0 ; i < data_list.size() ; i++) {
		    String s = (String)data_list.elementAt(i);
		    if (s.substring(0,2).equals(record.substring(0,2))) {
			data = s;
			break;
		    }
		}

		if (data.length() > 66) {
		    String name = "HassfortherV" + data.substring(0,2);
		    String ra_decl = data.substring(4,25);
		    String alias = "GSC:" + data.substring(27,37).trim();
		    alias += "  IRAS:" + data.substring(38,50).trim();
		    alias += "  USNO-A2.0:" + data.substring(51,65).trim();
		    alias += "  TASS:" + data.substring(66).trim();
		    String type = "Type:" + record.substring(4,12).trim();
		    String mag = record.substring(13,31).trim();

		    String dbvs_name = name.toUpperCase();

		    CoorMapping coor = new CoorMapping(Integer.parseInt(ra_decl.substring(0,2)), Integer.parseInt(ra_decl.substring(3,5)), Double.valueOf(ra_decl.substring(6,10)).doubleValue(), (ra_decl.charAt(12) == '-'), Integer.parseInt(ra_decl.substring(13,15)), Integer.parseInt(ra_decl.substring(16,18)), Double.valueOf(ra_decl.substring(19,21)).doubleValue() / 100 * 60);

		    record = name + "  " + ra_decl + "  " + mag + "  " + type + "  " + alias;

		    ReferenceData ref_data = new ReferenceData(name, dbvs_name, coor, record);

		    return ref_data;
		}
	    }
	}

	return null;
    }
}
