/*
 * IrasFsc.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
F00000+2252 0000004+225217 23  6 64        7   1.181E-012.405E-014.772E-011.069E+001131 29 29 13 2799              1.0E+01                     7.4E+00              40    0 0 0 0 0 0 0           1.35      0            1.600
F00000-0359 0000008-035924 16  4 67       12 121.561E-011.740E-011.121E+002.594E+001132 28 28  6  899              2.6E+011.7E+01              1.7E+012.9E+01       16 31 0 0 0 0 0 0 0           1.19 1.80 210          1.6001.610
 *
 */

package misao.ref.catalog;

import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class IrasFsc extends ReferenceCatalog {
    public IrasFsc ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.charAt(83) == ' '  ||  record.charAt(83) == '1')
	    return null;

	String name = "IRAS " + record.substring(0, 12).trim();
	String dbvs_name = "IRAS" + record.substring(0, 12).trim();

	CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(12,14).trim()), Integer.parseInt(record.substring(14,16).trim()), (double)Integer.parseInt(record.substring(16,19).trim()) / 10.0, (record.charAt(19) == '-'), Integer.parseInt(record.substring(20,22).trim()), Integer.parseInt(record.substring(22,24).trim()), (double)Integer.parseInt(record.substring(24,26).trim()));
	coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	String s = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl() + "  (" + record.substring(26,29).trim() + " x " + record.substring(29,32).trim() + "\", p.a.=" + record.substring(32,35).trim() + ")";

	String flux = record.substring(47,56).trim();
	if (flux.length() > 0)
	    s += "  flux(12)=" + Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	flux = record.substring(56,65).trim();
	if (flux.length() > 0)
	    s += "  flux(25)=" + Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	flux = record.substring(65,74).trim();
	if (flux.length() > 0)
	    s += "  flux(60)=" + Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	flux = record.substring(74,83).trim();
	if (flux.length() > 0)
	    s += "  flux(100)=" + Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
