/*
 * MisaoRev.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
Object     R.A. (J2000.0) Decl.   Max    Min   Type    Period      Epoch         ID
----------------------------------------------------------------------------------------------------
AQLAF      185932.17  -090658.5  11.2C  12.6C  ?            ?             ?      
AQLAG      185932.748 -060851.14 10.7C  11.9C  ?            ?             ?      USNO-A1.0_0825.13965291 IRAS18568-0613

Object     R.A. (J2000.0) Decl.  ID
------------------------------------------------------------
HYAGT      142650.720 -261644.15 USNO-A1.0_0600.09679667

Object                  R.A. (J2000.0) Decl.   Max    Min   Type    Period      Epoch         ID
-----------------------------------------------------------------------------------------------------------------
HS953                   175946.415 +000348.45 14.6C  15.4C  ?            ?             ?      USNO-A2.0_0900.11168813

Object                  R.A. (J2000.0) Decl.  ID
-------------------------------------------------------------------------
Hen1341                 170836.575 -172629.75 GSC6236.636 USNO-A1.0_0675.12060389
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class MisaoRev extends ReferenceCatalog {
    private boolean variable_flag;
    private boolean longname_flag;

    public MisaoRev ( ) {
	variable_flag = true;
	longname_flag = false;
    }

    public MisaoRev ( boolean _variable_flag, boolean _longname_flag ) {
	variable_flag = _variable_flag;
	longname_flag = _longname_flag;
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.length() > 0  &&  record.length() < 81)
	    variable_flag = false;

	if (record.length() < 6  ||  record.substring(0,6).equals("------")  ||  record.substring(0,6).equals("Object"))
	    return null;

	int offset = 0;
	if (longname_flag)
	    offset = 13;

	String name = record.substring(0, 10 + offset).trim();
	String dbvs_name = name;
	String RA = record.substring(11 + offset ,21 + offset).trim();
	String Decl = record.substring(22 + offset ,32 + offset).trim();
	String alias = "";
	if (variable_flag)
	    alias = record.substring(81 + offset).trim();
	else
	    alias = record.substring(33 + offset).trim();

	if (name.length() > 3) {
	    String cons = name.substring(0,3);
	    if (ConstellationTable.isConstellationCode(cons)) {
		String code = name.substring(3);
		name = code + " " + cons;
	    }
	}

	CoorMapping coor = new CoorMapping(Coor.createCoor(RA + " " + Decl));

	record = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl() + "  " + alias;

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

	return data;
    }

    public static ReferenceData updateData ( ReferenceData orig_data, ReferenceData patch_data ) {
	String orig_record = orig_data.getRecord();
	int p = orig_record.indexOf("Decl.");
	p = orig_record.indexOf("  ", p);
	if (p < 0)
	    p = orig_record.indexOf("Decl.") + 17;

	String patch_record = patch_data.getRecord();
	int q = patch_record.indexOf("Decl.");
	q = patch_record.indexOf("  ", p);
	if (q < 0)
	    q = patch_record.indexOf("Decl.") + 17;

	String record = patch_record.substring(0,q) + orig_record.substring(p);
	ReferenceData data = new ReferenceData(orig_data.getName(), orig_data.getDbvsName(), patch_data.getCoor(), record, orig_data.getPositionError(), orig_data.getType());
	return data;
    }
}
