/*
 * NovaList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
*** Nova ***
<1995>
Nova Cir 1995         01/27 14h44m53s.48  -63o53'55".6   7.2        N       W. Liller
Nova Aql 1995         02/07 19h05m26s.636 - 1o42'03".27  8.1   7.5  NB      Kesao Takamizawa
 *
 */

package misao.ref.catalog;

import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class NovaList extends ReferenceCatalog {
    String year;

    public NovaList ( ) {
	year = "";
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.charAt(0) == '<') {
	    year = record.substring(1, record.indexOf('>'));
	    return null;
	}

	if (record.charAt(0) == '*')
	    return null;

	double RAs = 0.0;
	if (record.charAt(34) == '.') {
	    RAs = (double)(Float.valueOf(record.substring(35,36).trim()).floatValue()) / 10.0 * 60.0;
	} else if (record.charAt(35) != ' ') {
	    RAs = (double)(Float.valueOf(record.substring(34,36).trim()).floatValue());
	    if (record.charAt(38) != ' ') {
		if (record.charAt(39) != ' ') {
		    RAs += (double)(Float.valueOf(record.substring(38,40).trim()).floatValue()) / 100.0;
		} else {
		    RAs += (double)(Float.valueOf(record.substring(38,39).trim()).floatValue()) / 10.0;
		}
	    }
	}

	double Decls = 0.0;
	if (record.charAt(49) == '.') {
	    Decls = (double)(Float.valueOf(record.substring(50,51).trim()).floatValue()) / 10.0 * 60.0;
	} else if (record.charAt(50) != ' ') {
	    Decls = (double)(Float.valueOf(record.substring(49,51).trim()).floatValue());
	    if (record.charAt(53) != ' ') {
		Decls += (double)(Float.valueOf(record.substring(53,54).trim()).floatValue()) / 10.0;
	    }
	}

	String name = record.substring(0, 22).trim();
	int p = record.indexOf(' ') + 1;
	int q = record.indexOf(' ', p) + 1;
	String tailer = name.substring(q).trim();
	String dbvs_name = record.substring(p, q - 1).toUpperCase() + "NOVA";
	for (int i = 0 ; i < tailer.length() ; i++) {
	    if (tailer.charAt(i) != ' ')
		dbvs_name += tailer.charAt(i);
	}

	String s = name;
	String ra = record.substring(28, 41).trim();
	if (ra.indexOf('.') > 0  &&  ra.charAt(ra.indexOf('.')) == ' ')
	    ra = ra.substring(0, ra.indexOf('.') - 1) + ra.substring(ra.indexOf('.'));
	String decl = record.substring(42, 55).trim();
	if (decl.indexOf('.') > 0  &&  decl.charAt(decl.indexOf('.')) == ' ')
	    decl = decl.substring(0, decl.indexOf('.') - 1) + decl.substring(decl.indexOf('.'));
	s += "  " + ra + "  " + decl + "  ";
	String mag = record.substring(56, 61).trim();
	if (mag.length() > 0)
	    s += mag + " mag";
	mag = record.substring(62, 67).trim();
	if (mag.length() > 0)
	    s += " (peak: " + mag + " mag)";
	String type = record.substring(68, 76).trim();
	if (type.length() > 0)
	    s += "  type:" + type;
	s += "  " + year + " " + JulianDay.getMonthCode(Integer.parseInt(record.substring(22, 24))) + " " + Integer.parseInt(record.substring(25, 27));
	s += "  " + record.substring(76);

	CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(28,30).trim()), Integer.parseInt(record.substring(31,33).trim()), RAs, (record.charAt(42) == '-'), Integer.parseInt(record.substring(43,45).trim()), Integer.parseInt(record.substring(46,48).trim()), Decls);

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
