/*
 * Nsv.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
00001   235934.8-670722       12.0     13.0     P 680780K BV 1184
00002  *000000. -790954:E:    14.9     15.7     P 340001  HV 6315      710726
00009  *                                                                      AT    Scl
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class Nsv extends ReferenceCatalog {
    public Nsv ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.charAt(8) != ' ') {
	    String name = "NSV" + record.substring(0, 5);
	    String dbvs_name = name;

	    double RAs = (double)Integer.parseInt(record.substring(12, 14));
	    if (record.charAt(15) != ' ')
		RAs += (double)Integer.parseInt(record.substring(15, 16)) / 10.0;

	    CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(8,10)), Integer.parseInt(record.substring(10,12)), RAs, (record.charAt(16) == '-'), Integer.parseInt(record.substring(17,19)), Integer.parseInt(record.substring(19,21)), (double)Integer.parseInt(record.substring(21,23)));
	    coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	    String s = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl();
	    String type = record.substring(24,30).trim();
	    if (type.length() > 0)
		s += "  type:" + type;
	    String mag = record.substring(30,35).trim();
	    if (mag.length() > 0) {
		s += "  " + mag;
		if (record.charAt(35) != ' ')
		    s += record.charAt(35);
	    }
	    mag = record.substring(37,39).trim() + record.substring(39,45).trim() + record.substring(45,47).trim() + record.substring(47,48).trim();
	    if (mag.length() > 0) {
		if (mag.charAt(0) == '(') {
		    s += " mag " + mag;
		} else {
		    s += " - " + mag + " mag";
		}
	    } else {
		s += " mag";
	    }
	    if (record.charAt(48) != ' ') {
		s += " (" + record.charAt(48) + ")";
	    }
	    if (record.length() >= 69) {
		String sp = "";
		if (record.length() >= 70)
		    sp = record.substring(68, 70).trim();
		else
		    sp = record.substring(68, 69).trim();
		if (sp.length() > 0)
		    s += "  Sp:" + sp;
	    }
	    if (record.length() >= 79) {
		String alias = record.substring(78).trim();
		s += " ";
		StringTokenizer st = new StringTokenizer(alias);
		while (st.hasMoreTokens())
		    s += " " + st.nextToken();
	    }

	    ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	    return data;
	}

	return null;
    }
}
