/*
 * NsvSupplement.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
15001*|000021.7-302542 |000255.2-300859|      |  8.40 |           |V |0002|  DM|A0V            |(A0)
15004*|000050.8+665149 |000326.0+670831|      | 13.6  |  15.0     |V |0003|0003|               |
15006*|000103.7-284144 |000337.1-282502|      |  8.21:|           |V |0004|    |A0p            |(B9)
15017 |000227.9-373611 |000524.4-372126|BY:   |  8.63 |(  0.02  ) |V |0008|  DM|M2V            |(K5)
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class NsvSupplement extends ReferenceCatalog {
    public NsvSupplement ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.length() < 95)
	    return null;

	String name = "NSV" + record.substring(0, 5);
	String dbvs_name = name;
	String RA = record.substring(24, 32);
	String Decl = record.substring(32, 39);
	String type = record.substring(40, 46).trim();
	String mag = record.substring(47, 54).trim() + " - " + record.substring(55, 66).trim();
	if (record.charAt(55) == '(') {
	    mag = record.substring(47, 54).trim() + " (" + record.substring(56, 64).trim() + ")";
	}
	String mag_code = record.substring(67, 69).trim();
	String spectrum = record.substring(80, 95).trim();

	if (RA.charAt(5) == ' ')
	    RA = RA.substring(0,5) + "0" + RA.substring(6);

	CoorMapping coor = new CoorMapping(Coor.createCoor(RA + " " + Decl));

	String s = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl();
	if (type.length() > 0)
	    s += "  type:" + type;
	if (mag.length() > 0) {
	    s += "  " + mag + " mag";
	    if (mag_code.length() > 0)
		s += " (" + mag_code + ")";
	}
	if (spectrum.length() > 0)
	    s += "  Sp:" + spectrum;

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
