/*
 * Array.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class Array {
    private double array[];
    private int size;

    private ArrayIndex sort_index;

    public Array ( int n ) {
	size = n;

	if (size > 0) {
	    array = new double[size];

	    for (int i = 0 ; i < size ; i++)
		array[i] = 0.0;
	}
    }

    public double getValue ( int index ) {
	if (0 <= index  &&  index < size)
	    return array[index];

	return 0.0;
    }

    public void set ( int index, double value ) {
	if (0 <= index  &&  index < size)
	    array[index] = value;
    }

    public void copy ( Array a ) {
	size = a.size;

	if (size > 0) {
	    array = new double[size];
	    for (int i = 0 ; i < size ; i++)
		set(i, a.getValue(i));
	}
    }

    public int getPeakIndex ( ) {
	if (size <= 0)
	    return -1;

	int index = 0;
	double value = getValue(0);
	for (int i = 1 ; i < size ; i++) {
	    if (value < getValue(i)) {
		index = i;
		value = getValue(i);
	    }
	}

	return index;
    }

    public int meanIndexOf ( double value ) {
	if (size <= 0)
	    return -1;

	double index = 0.0;
	int count = 0;
	for (int i = 0 ; i < size - 1 ; i++) {
	    if ((getValue(i) - value) * (getValue(i+1) - value) < 0.0) {
		index += (double)i;
		count++;
	    }
	}

	if (count == 0)
	    return -1;

	index /= (double)count;
	return (int)index;
    }

    public MinMaxAverage calculateMinMaxAverage ( ) {
	double min = 0.0;
	double max = 0.0;
	double average = 0.0;

	for (int i = 0 ; i < size ; i++) {
	    if (i == 0) {
		min = getValue(i);
		max = getValue(i);
	    } else {
		if (min > getValue(i))
		    min = getValue(i);
		if (max < getValue(i))
		    max = getValue(i);
	    }
	    average += getValue(i) / (double)size;
	}

	return new MinMaxAverage(min, max, average, size);
    }

    public void calculateMovingAverage ( int window_size ) {
	Array new_array = new Array(size);

	for (int i = 0 ; i < size ; i++) {
	    int base_index = i - (window_size - 1) / 2;
	    double value = 0.0;
	    int count = 0;
	    for (int j = 0 ; j < window_size ; j++) {
		if (0 <= base_index + j  &&  base_index + j < size) {
		    value += getValue(base_index + j);
		    count++;
		}
	    }
	    new_array.set(i, value / (double)count);
	}

	copy(new_array);
    }

    public void removeElement ( int start, int end ) {
	Array new_array = new Array(size - (end - start));
	int index = 0;

	for (int i = 0 ; i < start ; i++)
	    new_array.set(index++, getValue(i));
	for (int i = end ; i < size ; i++)
	    new_array.set(index++, getValue(i));

	copy(new_array);
    }

    public ArrayIndex sortAscendant ( ) {
	sort_index = null;

	if (size > 0) {
	    sort_index = new ArrayIndex(size);

	    qsort(0, size - 1, 0);

	    Array new_array = new Array(size);
	    for (int i = 0 ; i < size ; i++)
		new_array.set(i, getValue(sort_index.get(i)));
	    copy(new_array);
	}

	return sort_index;
    }

    public ArrayIndex sortDescendant ( ) {
	sort_index = null;

	if (size > 0) {
	    sort_index = new ArrayIndex(size);

	    qsort(0, size - 1, 1);

	    Array new_array = new Array(size);
	    for (int i = 0 ; i < size ; i++)
		new_array.set(i, getValue(sort_index.get(i)));
	    copy(new_array);
	}

	return sort_index;
    }

    // quick sort (flag 0: ascendant, 1: descendant)
    private void qsort ( int first, int last, int flag ) {
	double a = array[sort_index.get((first + last) / 2)];
	int i = first;
	int j = last;

	while (true) {
	    while (i < size  &&  ((flag == 0  &&  array[sort_index.get(i)] < a)  ||  (flag == 1  &&  array[sort_index.get(i)] > a)))
		i++;
	    while (j >= 0  &&  ((flag == 0  &&  array[sort_index.get(j)] > a)  ||  (flag == 1  &&  array[sort_index.get(j)] < a)))
		j--;
	    if (i >= j)
		break;

	    sort_index.swap(i, j);

	    i++;
	    j--;
	}

	if (first < i - 1)
	    qsort(first, i - 1, flag);
	if (last > j + 1)
	    qsort(j + 1, last, flag);
    }
}
