/*
 * Graph.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;

public class Graph {
    // branch[i][j] is true if node i and j is directly connected
    boolean[][] branch;

    int N;

    // for temporary use
    boolean[] visit_flag;
    int max_steps;

    public Graph ( int n ) {
	branch = new boolean[n][n];
	for (int i = 0 ; i < n ; i++) {
	    for (int j = 0 ; j < n ; j++)
		branch[i][j] = false;
	}

	N = n;
    }

    public int getNodeCount ( ) {
	return N;
    }

    public void connect ( int i, int j ) {
	branch[i][j] = true;
	branch[j][i] = true;
    }

    public void disconnect ( int i, int j ) {
	branch[i][j] = false;
	branch[j][i] = false;
    }

    public boolean isConnected ( int i, int j ) {
	return branch[i][j];
    }

    public boolean isReachable ( int i, int j ) {
	int[] reachable_index = getIndexListReachable(i);
	for (int k = 0 ; k < reachable_index.length ; k++) {
	    if (j == reachable_index[k])
		return true;
	}
	return false;
    }

    public boolean isReachableInSteps ( int i, int j, int steps ) {
	int[] reachable_index = getIndexListReachableInSteps(i, steps);
	for (int k = 0 ; k < reachable_index.length ; k++) {
	    if (j == reachable_index[k])
		return true;
	}
	return false;
    }

    // return sorted index
    public int[] getIndexListReachable ( int i ) {
	return getIndexListReachableInSteps ( i, -1 );
    }

    // return sorted index
    public int[] getIndexListReachableInSteps ( int i, int steps ) {
	max_steps = steps;

	visit_flag = new boolean[getNodeCount()];
	for (int k = 0 ; k < getNodeCount() ; k++)
	    visit_flag[k] = false;

	getIndexListReachableInStepsRecurrent(i, 0);

	int count = 0;
	for (int k = 0 ; k < getNodeCount() ; k++) {
	    if (visit_flag[k])
		count++;
	}

	int[] list = new int[count];
	count = 0;
	for (int k = 0 ; k < getNodeCount() ; k++) {
	    if (visit_flag[k]) {
		list[count] = k;
		count++;
	    }
	}

	return list;
    }

    private void getIndexListReachableInStepsRecurrent ( int i, int current_step ) {
	visit_flag[i] = true;

	if (current_step == max_steps)
	    return;

	for (int j = 0 ; j < getNodeCount() ; j++) {
	    if (i != j  &&  isConnected(i,j)  &&  visit_flag[j] == false)
		getIndexListReachableInStepsRecurrent(j, current_step + 1);
	}
    }

    public void printConnectedTable ( PrintStream ps ) {
	for (int i = 0 ; i < getNodeCount() ; i++) {
	    for (int j = 0 ; j < getNodeCount() ; j++) {
		if (isConnected(i,j)) {
		    ps.print("  T");
		} else {
		    ps.print("  .");
		}
	    }
	    ps.println("");
	}
    }

    public void printReachableTable ( PrintStream ps ) {
	for (int i = 0 ; i < getNodeCount() ; i++) {
	    for (int j = 0 ; j < getNodeCount() ; j++) {
		if (isReachable(i,j)) {
		    ps.print("  T");
		} else {
		    ps.print("  .");
		}
	    }
	    ps.println("");
	}
    }
}
