/*
 * ImageFile.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.net.*;
import java.awt.*;
import misao.secret.util.*;

public class ImageFile {
    private ImageGray gray_image;
    private ImageRGB rgb_image;

    public ImageFile ( URL url ) throws Exception {
	ImageType type = new ImageType(url);

	gray_image = null;
	rgb_image = null;

	switch ( type.getType() ) {
	    case ImageType.IMAGE_GIF:
		rgb_image = new ImageFileGif(url);
		break;
	    case ImageType.IMAGE_JPEG:
		rgb_image = new ImageFileJpeg(url);
		break;
	    case ImageType.IMAGE_PGM:
		gray_image = new ImageFilePgm(url);
		break;
	    case ImageType.IMAGE_PPM:
		rgb_image = new ImageFilePpm(url);
		break;
	    case ImageType.IMAGE_FITS:
		gray_image = new ImageFileFits(url);
		break;
	    case ImageType.IMAGE_MTF:
		gray_image = new ImageFileMtf(url);
		break;
	    case ImageType.IMAGE_BIN:
		gray_image = new ImageFileBin(url);
		break;
	}
    }

    public int getWidth ( ) {
	if (gray_image != null)
	    return gray_image.getWidth();
	else if (rgb_image != null)
	    return rgb_image.getWidth();

	return 0;
    }

    public int getHeight ( ) {
	if (gray_image != null)
	    return gray_image.getHeight();
	else if (rgb_image != null)
	    return rgb_image.getHeight();

	return 0;
    }

    public Size getSize( ) {
	if (gray_image != null)
	    return gray_image.getSize();
	else if (rgb_image != null)
	    return rgb_image.getSize();

	return new Size(0, 0);
    }

    public MinMaxAverage calculateMinMaxAverage ( ) {
	if (gray_image != null)
	    return gray_image.calculateMinMaxAverage();
	else if (rgb_image != null)
	    return rgb_image.calculateMinMaxAverage();

	return new MinMaxAverage();
    }

    public ImageGray getGrayImage ( ) throws Exception {
	if (gray_image != null)
	    return gray_image;
	else if (rgb_image != null)
	    return rgb_image.getGrayImage();

	return null;
    }

    public Image getImage ( ) throws Exception {
	if (gray_image != null)
	    return gray_image.getImage();
	else if (rgb_image != null)
	    return rgb_image.getImage();

	return null;
    }

    public Image getImage ( double min, double max ) throws Exception {
	if (gray_image != null)
	    return gray_image.getImage(min, max);
	else if (rgb_image != null)
	    return rgb_image.getImage(min, max);

	return null;
    }

    // left <-> right
    public void mirrorHorizontal ( ) {
	if (gray_image != null)
	    gray_image.mirrorHorizontal();
	if (rgb_image != null)
	    rgb_image.mirrorHorizontal();
    }

    // upside down
    public void mirrorVertical ( ) {
	if (gray_image != null)
	    gray_image.mirrorVertical();
	if (rgb_image != null)
	    rgb_image.mirrorVertical();
    }
}
