/*
 * ImageType.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;
import java.net.*;

public class ImageType {
    public static final int IMAGE_GIF = 0;
    public static final int IMAGE_JPEG = 1;
    public static final int IMAGE_PGM = 2;
    public static final int IMAGE_PPM = 3;
    public static final int IMAGE_FITS = 4;
    public static final int IMAGE_MTF = 5;
    public static final int IMAGE_BIN = 6;

    private URL url;

    public ImageType ( URL u ) {
	url = u;
    }

    public int getType ( ) throws Exception {
	URLConnection uc;
	try {
	    uc = url.openConnection();
	} catch ( IOException e ) {
	    System.err.println("Cannot connect to: " + url.toString());
	    throw e;
	}

	String mimeType = uc.getContentType();

	String s = url.toString();

	if (mimeType.equals("image/gif")) {
	    return IMAGE_GIF;
	} else if (mimeType.equals("image/jpeg")) {
	    return IMAGE_JPEG;
	} else if (mimeType.equals("image/x-portable-graymap")) {
	    return IMAGE_PGM;
	} else if (mimeType.equals("image/x-portable-pixmap")) {
	    return IMAGE_PPM;
	} else if (s.substring(s.length() - 4).equalsIgnoreCase(".fts")  ||  
		   s.substring(s.length() - 4).equalsIgnoreCase(".fit")  ||  
		   s.substring(s.length() - 5).equalsIgnoreCase(".fits")) {
	    return IMAGE_FITS;
	} else if (s.substring(s.length() - 4).equalsIgnoreCase(".mtf")) {
	    return IMAGE_MTF;
	} else if (s.substring(s.length() - 4).equalsIgnoreCase(".bin")) {
	    return IMAGE_BIN;
	} else {
	    System.err.println("Not an image [" + mimeType + "]: " + url.toString());
	    throw new Exception();
	}
    }

    public boolean isGif ( ) {
	try {
	    return (getType() == IMAGE_GIF);
	} catch ( Exception e ) {
	    return false;
	}
    }

    public boolean isJpeg ( ) {
	try {
	    return (getType() == IMAGE_JPEG);
	} catch ( Exception e ) {
	    return false;
	}
    }

    public boolean isPgm ( ) {
	try {
	    return (getType() == IMAGE_PGM);
	} catch ( Exception e ) {
	    return false;
	}
    }

    public boolean isPpm ( ) {
	try {
	    return (getType() == IMAGE_PPM);
	} catch ( Exception e ) {
	    return false;
	}
    }

    public boolean isFits ( ) {
	try {
	    return (getType() == IMAGE_FITS);
	} catch ( Exception e ) {
	    return false;
	}
    }

    public boolean isMtf ( ) {
	try {
	    return (getType() == IMAGE_MTF);
	} catch ( Exception e ) {
	    return false;
	}
    }

    public boolean isBin ( ) {
	try {
	    return (getType() == IMAGE_BIN);
	} catch ( Exception e ) {
	    return false;
	}
    }
}
