/*
 * InfoCanvas.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.awt.*;
import java.util.*;

public class InfoCanvas extends Canvas {
    private Size size;
    private String title;
    private Vector string_list;
    private boolean frame_line;

    private static final int fontsize = 12;
    private static final int fontwidth= 7;
    private static final int lfontsize = 18;
    private static final int lfontwidth= 10;

    public InfoCanvas ( Size s ) {
	size = s;
	title = "";
	string_list = new Vector();
	frame_line = true;

	resize(size.getWidth(), size.getHeight());
	setBackground(Color.white);
    }

    public void clear ( ) {
	title = "";
	string_list = new Vector();
    }

    public void setTitle ( String s ) {
	title = s;
    }

    public void addString ( String s ) {
	string_list.addElement(s);
    }

    public void addString ( String[] s ) {
	string_list.addElement(s);
    }

    public void setFramelineOff ( ) {
	frame_line = false;
    }

    public void paint ( Graphics g ) {
	update(g);
    }

    public void update ( Graphics g ) {
	int y = lfontsize;

	// clear
	g.setColor(getBackground());
	g.fillRect(0, 0, size.getWidth(), size.getHeight());

	// frame line
	if (frame_line) {
	    g.setColor(Color.gray);
	    g.drawRect(0, 0, size.getWidth() - 1, size.getHeight() - 1);
	}

	// titile
	g.setFont(new Font("TimesRoman", Font.BOLD, lfontsize));
	g.setColor(Color.black);
	g.drawString(title, 0, y);
	y += lfontsize + 4;

	// data
	g.setFont(new Font("TimesRoman", Font.PLAIN, fontsize));
	for (int i = 0 ; i < string_list.size() ; i++) {
	    if (string_list.elementAt(i) instanceof String) {
		String s = (String)string_list.elementAt(i);
		g.drawString(s, 4, y);
	    } else {
		String[] s = (String[])string_list.elementAt(i);
		for (int j = 0 ; j < s.length ; j++) {
		    int x = 4 + j * size.getWidth() / s.length;
		    g.drawString(s[j], x, y);
		}
	    }
	    y += fontsize + 2;
	}
    }
}
